﻿Imports System.ComponentModel
Imports System.IO
Imports System.Net
Imports System.Net.Http
Imports Microsoft.Win32

Class MainWindow
	Implements INotifyPropertyChanged

	Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
	Public ReadOnly Property FilteredComponents As IEnumerable(Of Component)
		Get
			Dim query = txtQuery.Text.ToLowerInvariant()
			If query.Trim() = "" Then
				Return Components
			End If

			Return From x In Components
				   Where x.Name.ToLowerInvariant().Contains(query)
		End Get
	End Property
	Private ReadOnly Property Components As New List(Of Component)

	Private Sub LoadDatabase(file As String)
		Dim lineNumber As Integer = 1

		Using sr As New StreamReader(file)
			While Not sr.EndOfStream
				Dim line = sr.ReadLine()
				If line.Trim() = "" Then
					Continue While
				End If

				Dim parts = line.Split(",")

				If parts.Length < 3 Then
					Throw New Exception($"Database file is invalid. Line {lineNumber} is missing component postion coordinates.")
				End If

				Dim xPosString = parts(parts.Length - 1)
				Dim yPosString = parts(parts.Length - 2)
				Dim partName = String.Join(",", parts.Take(parts.Length - 2))

				Dim xPos As Integer
				Dim yPos As Integer

				If Not Integer.TryParse(xPosString, xPos) Then
					Throw New Exception($"Database file is invalid. Line {lineNumber} has non-numeric coordinate.")
				End If

				If Not Integer.TryParse(yPosString, yPos) Then
					Throw New Exception($"Database file is invalid. Line {lineNumber} has non-numeric coordinate.")
				End If

				If xPos < 0 Or yPos < 0 Or xPos > 400 Or yPos > 400 Then
					Throw New Exception($"Database file is invalid. Line {lineNumber} has coordinate value out of allowed range.")
				End If

				Components.Add(New Component(partName, xPos, yPos))

				lineNumber += 1
			End While
		End Using
	End Sub

	Private Sub MainWindow_Loaded(sender As Object, e As RoutedEventArgs) Handles Me.Loaded
		Dim ofd As New OpenFileDialog()
		ofd.Filter = "CSV Components Database (.csv)|*.csv|All files|*"
		If ofd.ShowDialog() Then
			Try
				LoadDatabase(ofd.FileName)
				DataContext = Me
				' RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(NameOf(FilteredComponents)))
			Catch ex As Exception
				MessageBox.Show("Error while loading database file. Details:" & vbCrLf & vbCrLf & ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error)
				End
			End Try
		Else
			End
		End If
	End Sub

	Private Sub txtQuery_TextChanged(sender As Object, e As TextChangedEventArgs)
		RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(NameOf(FilteredComponents)))
	End Sub

	Private Sub Button_Click(sender As Button, e As RoutedEventArgs)
		Dim component = CType(sender.Tag, Component)
		Dim cli = New HttpClient()
		Try
			If txtIp.Text.Trim() = "" Then
				Throw New Exception("Missing Storage Box IP address in top textbox.")
			End If

			Dim response = cli.GetAsync("http://" & txtIp.Text & $"/reach/{component.PositionX}/{component.PositionY}").GetAwaiter().GetResult()
			If response.StatusCode <> 200 Then
				Throw New Exception($"Storage Box responded with error status code {response.StatusCode}.")
			End If
		Catch ex As Exception
			Return
			MessageBox.Show("Error while opening the box. Details:" & vbCrLf & vbCrLf & ex.Message, "Error", MessageBoxButton.OK, MessageBoxImage.Error)
		End Try
	End Sub
End Class
